/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.allgemein;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.MutableCollection;
import de.bsvrz.dav.daf.main.config.ObjectSetType;
import de.bsvrz.dav.daf.main.config.ObjectTimeSpecification;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Klasse zur Verwaltung von Objektlisten, typischerweise alle Objekte eines Typs, sowie
 * zusätzlich feste Attributgruppe und Aspekt.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class ObjekteAtgAsp implements Comparator<ObjekteAtgAsp> {
  /** Verbindung zum Datenverteiler. */
  private ClientDavInterface davConnection;
  
  /** Comparator-Klasse: Vergleich über die Pid eines Datenverteiler-Objekts. */
  public class ComparatorPid implements Comparator<SystemObject> {
    @Override
    public int compare(final SystemObject o1, final SystemObject o2) {
      return o1.getPid().compareTo(o2.getPid());
    }
  }
  
  /** Liste der Datenverteiler-Objekte. */
  private List<SystemObject> systemObjekte;

  /** Collection von dynamischen Objekten. */  
  private MutableCollection mutableCollection;
  
  /** Typ der Datenverteiler-Objekte (bei direkter Veränderung der Objekt-Liste sind Inkonsistenzen möglich!). */
  private SystemObjectType typ;
  
  /** Attribut-Gruppe. */
  private AttributeGroup attributGruppe;
  
  /** Aspekt. */
  private Aspect aspekt;
  
  /** Aus Attribut-Gruppe und Aspekt gebildete DataDescription. */
  private DataDescription dataDescription;
  
  /**
   * Konstruktor. Die Felder (Typ, Atg, Asp) bleiben leer.
   * 
   * @param davConnection Datenverteiler-Verbindung
   */
  public ObjekteAtgAsp(final ClientDavInterface davConnection) {
    this.davConnection = davConnection;
  }

  /**
   * Konstruktor. Setzt nur den Typ der Sammlung.
   * 
   * @param davConnection Datenverteiler-Verbindung
   * @param typ Pid eines Datenverteiler-Typs
   */
  public ObjekteAtgAsp(final ClientDavInterface davConnection, final String typ) {
    this.davConnection = davConnection;
    setTyp(typ);
  }
  
  /**
   * Konstruktor. Setzt neben Typ auch Attributgruppe und Aspekt der Sammlung.
   * 
   * @param davConnection Datenverteiler-Verbindung
   * @param typ Pid eines Datenverteiler-Typs
   * @param atg Pid einer Attributgruppe
   * @param asp Pid eines Aspekts
   */
  public ObjekteAtgAsp(final ClientDavInterface davConnection, final String typ, final String atg, final String asp) {
    this.davConnection = davConnection;
    setTyp(typ);
    setAttributGruppe(atg);
    setAspekt(asp);
  }
  
  /**
   * Konstruktor. Setzt Typ und resultierende Objekte aus einer dynamischen Menge heraus. Die
   * Attributgruppe und der Aspekt beziehen sich auf die resultierenden dynamischen Objekte.
   * 
   * Die Menge darf nur genau einen Typ definieren.
   * 
   * @param davConnection Datenverteiler-Verbindung
   * @param obj Objekt, dass die Menge definiert
   * @param mengenName Name der Menge
   * @param atg Pid einer Attributgruppe
   * @param asp Pid eines Aspekts
   */
  public ObjekteAtgAsp(final ClientDavInterface davConnection, final ConfigurationObject obj, 
      final String mengenName, final String atg, final String asp) {
    this.davConnection = davConnection;
    mutableCollection = obj.getMutableSet(mengenName);
    ObjectSetType setType = obj.getObjectSet(mengenName).getObjectSetType();

    if (setType.getObjectTypes().size() != 1) {
      throw new IllegalArgumentException("Nicht implementierte Funktionalität: Die Menge '" 
          + mutableCollection.getType().getPidOrNameOrId() + "' definiert mehr als einen Typ.");   
    }

    this.typ = setType.getType();
    
    setAttributGruppe(atg);
    setAspekt(asp);
  }
  
  /**
   * Zugriff auf Liste mit Systemobjekten. Wenn diese bislang nicht initialisiert wurde,
   * hole alle zum definierten Datenverteiler-Typ gehörigen Objekte.
   * 
   * @return Liste mit Datenverteiler-Objekten
   */
  
  public List<SystemObject> getSystemObjekte() {
    if (mutableCollection != null) {
      return mutableCollection.getElements((short) 0);
    } else {
      if (systemObjekte == null) {
        systemObjekte = new ArrayList<>();
        holeSystemObjekte();
      }
      return systemObjekte;
    }
  }
  

  /**
   * Liefert die MutableCollection (Liste der dynamischen Objekte) zurück.
   * 
   * @return MutableCollection-Instanz oder null, wenn nicht ObjekteAtgAsp-Instanz nicht für dynamische
   *         Objekte instantiiert wurde.
   */
  
  public MutableCollection getMutableCollection() {
    return mutableCollection;
  }
  
  /**
   * Definierter Typ als Datenverteiler-Objekt.
   * 
   * @return Typ
   */
  
  public SystemObjectType getTyp() {
    return typ;
  }

  /**
   * Setzte Typ als Pid eines Datenverteiler-Typs. Die Liste der Objekte wird dabei neu initialisiert. 
   * 
   * @param typName Pid des Typs
   */
  public void setTyp(final String typName) {
    if (systemObjekte != null) {
      systemObjekte.clear();
    }
    this.typ = davConnection.getDataModel().getType(typName);
  }
  
  /**
   * Füge alle Datenverteiler-Objekte vom in der Klasse gesetzen Typ der Objekt-Liste hinzu.
   * 
   */
  
  public void holeSystemObjekte() {
    Collection<SystemObjectType> listeTypen = new ArrayList<>();
    listeTypen.add(typ);
    DataModel dm = davConnection.getDataModel();
    dm = davConnection.getDataModel();

    Collection<SystemObject> clist = dm.getObjects(null, listeTypen, ObjectTimeSpecification.valid());
    if (clist instanceof List) {
      systemObjekte = (List<SystemObject>) clist;
    } else {
      systemObjekte.clear();
      systemObjekte.addAll(clist);
    }
  }
  
  /**
   * Sortiere die Objekt-Liste nach der Pid der Objekte.
   */
  
  public void sortiereObjekte() {
    Collections.sort(getSystemObjekte(), new ComparatorPid());
  }
  
  /**
   * Gibt die zuvor definierte Attributgruppe zurück.
   * 
   * @return Attributgruppe
   */

  public AttributeGroup getAttributGruppe() {
    return attributGruppe;
  }

  /**
   * Gibt den zuvor definierte Aspekt zurück.
   * 
   * @return Aspect
   */
  
  public Aspect getAspekt() {
    return aspekt;
  }
  
  /**
   * Gibt den zu Attributgruppe und Aspekt passenden DataDescription zurück.
   * 
   * @return DataDescription
   */
  
  public DataDescription getDataDescription() {
    return dataDescription;
  }
  
  /**
   * Setzte Attributgruppe durch Angabe der Pid einer solchen.
   * 
   * @param atg Pid einer Attributgruppe
   */

  public void setAttributGruppe(final String atg) {
    this.attributGruppe = davConnection.getDataModel().getAttributeGroup(atg);
    this.dataDescription = new DataDescription(attributGruppe, aspekt);
  }
  
  /**
   * Setzte Aspekt durch Angabe der Pid einer solchen.
   * 
   * @param asp Pid des Aspekts
   */

  public void setAspekt(final String asp) {
    this.aspekt = davConnection.getDataModel().getAspect(asp);
    this.dataDescription = new DataDescription(attributGruppe, aspekt);
  }

  /**
   * {@inheritDoc}
   * Zwei Objekte von Typ ObjekteAtgAsp sind gleich, wenn der definierte Typ, Attributgruppe und Aspekt gleich sind.
   * Insbesondere aber ist die tatsächliche Objekt-Liste irrelevant.
   * 
   * @return true: es wird die gleiche Liste bezeichnet
   */

  @Override
  public boolean equals(final Object object) {
    if (object instanceof ObjekteAtgAsp) {
      ObjekteAtgAsp o = (ObjekteAtgAsp) object;
      if (this.typ == null) {
        if (o.typ != null) {
          return false;
        }
      } else {
        if (!this.typ.equals(o.typ)) {
          return false;
        }
      }
      if (this.attributGruppe == null) {
        if (o.attributGruppe != null) {
          return false;
        }
      } else {
        if (!this.attributGruppe.equals(o.attributGruppe)) {
          return false;
        }
      }
      if (this.aspekt == null) {
        if (o.aspekt != null) {
          return false;
        }
      } else {
        if (!this.aspekt.equals(o.aspekt)) {
          return false;
        }
      }
      return true; 
    } else {
      return false;
    }
  }
  
  @Override
  public int hashCode() {
    return ((this.typ != null) ? this.typ.hashCode() : 0) 
        + ((this.attributGruppe != null) ? this.attributGruppe.hashCode() : 0)
        + ((this.aspekt != null) ? this.aspekt.hashCode() : 0);
  }
  
  /**
   * {@inheritDoc}
   * Definiere Ordnung auf ObjekteAtgAsp: Sortierung nach Pid von Typ, dann Pid von Attributgruppe, dann Pid von Aspekt.
   */
  
  @Override
  public int compare(final ObjekteAtgAsp o1, final ObjekteAtgAsp o2) {
    int result = o1.getTyp().getPid().compareTo(o2.getTyp().getPid());
    if (result == 0) {
      result = o1.getAttributGruppe().getPid().compareTo(o2.getAttributGruppe().getPid());
    }
    if (result == 0) {
      result = o1.getAspekt().getPid().compareTo(o2.getAspekt().getPid());      
    }
    return result;
  }
  
}
