/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.definition;
import java.util.ArrayList;
import java.util.Collection;

import javax.naming.ConfigurationException;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.kex.datex2.allgemein.ErweiterterLogger;
import de.bsvrz.kex.datex2.allgemein.KonfigurationsModul;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.allgemein.PersistenzModul;
import de.bsvrz.kex.datex2.plugins.ausfuehrung.PluginAusfuehrungsAlgorithmus;

/**
 * Schnittstelle zur Plugin-Definitions-Klasse.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public abstract class KExD2Plugin {
  /** Algorithmus zur Ausführung des PlugIns. */
  protected PluginAusfuehrungsAlgorithmus pluginAusfuehrungsAlgorithmus;
  
  /** Referenz auf PersistenzModul. */
  protected PersistenzModul persistenzModul;
  
  /** Referenz auf KonfigurationsModul. */
  protected KonfigurationsModul konfigurationsModul;
  
  /** Referenz auf Datenverteiler-Verbindung. */
  protected ClientDavInterface davConnection;
  
  /** Referenz auf erweiterten Logger.*/
  protected ErweiterterLogger erweiterterLogger;
  
  /** Liste der zu behandelnen Objekte nebst Attributgruppen und Aspekten. */
  protected Collection<ObjekteAtgAsp> objekte = new ArrayList<>();
  
  /** Periode der Ausführung (Publikation dynamischer Daten). */
  protected int period;
  
  /**
   * Konstruktor.
   * 
   * @param standardPeriod Standard-Wert für Periode.
   */
  
  protected KExD2Plugin(final int standardPeriod) {
    this.period = standardPeriod;
  }
  
  /**
   * Rückgabe aller zu behandelnden Objektlisten/Attributgruppen/Aspekt-Kombíntationen.
   * @return Liste ObjekteAtgAsp
   */
  
  public Collection<ObjekteAtgAsp> getObjects() {
    return objekte;
  }
  
  /**
   * Rückgabe der Ausführungsperiode für dynamische Daten (in Sekunden).
   * @return Periode in Sekunden
   */

  public final long getPeriod() {
    return period;
  }
  
  /**
   * Initialisiere das Plugin.
   * 
   * @throws ConfigurationException Fehlerhafte Konfiguration
   */
  
  public abstract void initialisiere() throws ConfigurationException;

  /**
   * Prüft, ob Plugin ordungsgemäß initialisiert wurde.
   * 
   * @return true, wenn ordnungsgemäß initialisiert
   */
  public abstract boolean isInitialisiert();

  /**
   * Injizierte KonfigurationsModul (mit bestimmten Kontext).
   * 
   * @param konfigurationsModul Referenz auf KonfigurationsModul
   */
  
  public void setKonfigurationsModul(final KonfigurationsModul konfigurationsModul) {
    this.konfigurationsModul = konfigurationsModul;
  }
  /**
   * Injizierte PersistenzModul (mit bestimmten Kontext).
   * 
   * @param persistenzModul Referenz auf PersistenzModul
   */
  
  public void setPersistenzModul(final PersistenzModul persistenzModul) {
    this.persistenzModul = persistenzModul;
  }
  
  /**
   * Injizierte Datenverteiler-Verbindung.
   * 
   * @param davConnection Referenz auf Datenverteiler-Verbindung
   */
  
  public void setDaVConnection(final ClientDavInterface davConnection) {
    this.davConnection = davConnection;
  }
  
  /**
   * Zugriff auf das zum Plugin assoziierte Konfigurationsmodul.
   * 
   * @return Referenz auf KonfigurationsModul
   */
  
  public KonfigurationsModul getKonfigurationsModul() {
    return konfigurationsModul;
  }

  /**
   * Zugriff auf das zum Plugin assoziierte Persistenzmodul.
   * 
   * @return Referenz auf PersistenzModul
   */
  
  public PersistenzModul getPersistenzModul() {
    return persistenzModul;
  }

  /**
   * Zugriff auf Datenverteiler-Verbindung.
   * 
   * @return Referenz auf Datenverteiler-Verbindung
   */
  
  public ClientDavInterface getDaVConnection() {
    return davConnection;
  }

  /**
   * Zugriff auf Algorithm zur Ausführung des PlugIns.
   *
   * @return Referenz auf Algorithmus-Objekt
   */
  
  public PluginAusfuehrungsAlgorithmus getPluginAusfuehrungsAlgorithmus() {
    return pluginAusfuehrungsAlgorithmus;
  }
  
  /**
   * Zugriff auf erweiterter Logger.
   * 
   * @return Referenz auf Logger-Objekt
   */
  
  public ErweiterterLogger getErweiterterLogger() {
    return erweiterterLogger;
  }

  /**
   * Injizierte erweiterten Logger.
   * 
   * @param erweiterterLogger Referenz auf erweiterter Logger.
   */
  
  public void setErweiterterLogger(final ErweiterterLogger erweiterterLogger) {
    this.erweiterterLogger = erweiterterLogger;
  }  
}