/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.definition.impl;

import javax.naming.ConfigurationException;

import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.ausfuehrung.impl.StandardExportPluginAusfuehrung;
import de.bsvrz.kex.datex2.plugins.datensammlung.impl.ImmerLetztesElement;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.Baustellen;
import de.bsvrz.kex.datex2.plugins.publizierung.impl.MDMPublizierung;
import de.bsvrz.kex.datex2.plugins.publizierung.impl.XMLFilePublizierung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Plugin-Definitions-Klasse ("assembly class") für das Plugin DaV-MDM-011
 * (Baustellen).
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */

public class DaV_MDM_011 extends PluginExport {

	/**
	 * Name des Konfigurationseintrages f&uuml;r das
	 * MDM-Baustellenverwaltungsobjekt.
	 */
	private static final String MDM_OBJEKT_NAME = "MDM-Baustellenverwaltung";

	/** Typ des MDM-Baustellenverwaltungsobjektes. */
	private static final String MDM_OBJEKT_TYP = "typ.mdmBaustellenVerwaltung";

	/** Name der Menge fuer die Baustellen. */
	private static final String BAUSTELLEN_MENGE = "MengeMdmBaustellen";

	/** Zu behandelnde Attributgruppe. */
	private static final String BAUSTELLE_DATEN_ATG = "atg.mdmBaustellenDatenGesamt";

	/** Zu behandelnder Aspekt. */
	private static final String BAUSTELLE_DATEN_ASP = "asp.parameterSoll";

	/** Logging. */
	private static final Debug LOGGER = Debug.getLogger();

	/** Standard-Periode des Plugins (in Sekunden). */
	private static final int STANDARD_PERIOD = 300;

	/** Mindestversion KB "kb.tmMdmBaustellen". */
	private static final int KB_MDM_MINDEST_VERION = 5;

	/**
	 * Konstruktor.
	 */
	public DaV_MDM_011() {
		super(STANDARD_PERIOD);
	}

	@Override
	public void initialisiere() throws ConfigurationException {
		LOGGER.finer("Plugin-Klasse " + this.getClass().getName() + " wird initialisiert.");
		ConfigurationObject mdmBaustellenVerwaltung = null;
		final String mdmObjektString = konfigurationsModul.getString(MDM_OBJEKT_NAME);
		final String logPrefix = "In Kontext '" + konfigurationsModul.getContext() + "': ";
		if (mdmObjektString == null) {
			throw new ConfigurationException(
					logPrefix + "Konfigurationsdatum '" + MDM_OBJEKT_NAME + "' wurde nicht angegeben.");
		}
		try {
			mdmBaustellenVerwaltung = (ConfigurationObject) davConnection.getDataModel().getObject(mdmObjektString);
			if (mdmBaustellenVerwaltung == null) {
				throw new ConfigurationException(logPrefix + "Das angegebene MDM-Baustellenverwaltungsobjekt '"
						+ mdmObjektString + "' konnte nicht gefunden werden.");
			}
		} catch (final ClassCastException ex) {
			throw new ConfigurationException(logPrefix + "Das angegebene MDM-Baustellenverwaltungsobjekt '"
					+ mdmObjektString + "' ist kein Konfigurationsobjekt.");
		}

		if (!mdmBaustellenVerwaltung.isOfType(MDM_OBJEKT_TYP)) {
			throw new ConfigurationException(logPrefix + "Das angegebene MDM-Baustellenverwaltungsobjekt '"
					+ mdmObjektString + "' ist nicht vom Typ '" + MDM_OBJEKT_TYP + "'!");
		}

		final ConfigurationArea configurationArea = davConnection.getDataModel()
				.getConfigurationArea("kb.tmMdmBaustellen");
		if (configurationArea == null) {
			throw new ConfigurationException(
					logPrefix + "Der Konfigurationsbereich 'kb.tmMdmBaustellen' ist nicht aktiviert.");
		}
		final short activeVersion = configurationArea.getActiveVersion();
		if (activeVersion < KB_MDM_MINDEST_VERION) {
			throw new ConfigurationException(
					logPrefix + "Der Konfigurationsbereich 'kb.tmMdmBaustellen' muss mindestens in der Version "
							+ KB_MDM_MINDEST_VERION + " aktiviert sein.");
		}

		final ObjekteAtgAsp rds = new ObjekteAtgAsp(davConnection, mdmBaustellenVerwaltung, BAUSTELLEN_MENGE,
				BAUSTELLE_DATEN_ATG, BAUSTELLE_DATEN_ASP);
		objekte.add(rds);

		initialisiereAbhaengigkeiten();
	}

	@Override
	public void instantiiereDynamischePublizierungsMethode() {
		dynamischePublizierungsMethoden.add(new XMLFilePublizierung(this, "dynamisch"));
		dynamischePublizierungsMethoden.add(new MDMPublizierung(this, "dynamisch"));
	}

	@Override
	public void instantiiereStatischePublizierungsMethode() {
	}

	@Override
	public void instantiiereDatexKonvertierungStatisch() {
		this.datexKonvertierungStatisch = null;
	}

	@Override
	public void instantiiereDatexKonvertierungDynamisch() {
		this.datexKonvertierungDynamisch = new Baustellen(this);
	}

	@Override
	public void instantiierePluginAusfuehrungsAlgorithmus() {
		this.pluginAusfuehrungsAlgorithmus = new StandardExportPluginAusfuehrung(this);
	}

	@Override
	public void instantiiereDatenSammlungsMethode() {
		this.datenSammlungsMethode = new ImmerLetztesElement(this);
	}
}
