/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.datatype.XMLGregorianCalendar;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import eu.datex2.schema.dynamisch.baustellen.ConfidentialityValueEnum;
import eu.datex2.schema.dynamisch.baustellen.CountryEnum;
import eu.datex2.schema.dynamisch.baustellen.D2LogicalModel;
import eu.datex2.schema.dynamisch.baustellen.Exchange;
import eu.datex2.schema.dynamisch.baustellen.HeaderInformation;
import eu.datex2.schema.dynamisch.baustellen.InformationStatusEnum;
import eu.datex2.schema.dynamisch.baustellen.InternationalIdentifier;
import eu.datex2.schema.dynamisch.baustellen.ObjectFactory;
import eu.datex2.schema.dynamisch.baustellen.PayloadPublication;
import eu.datex2.schema.dynamisch.baustellen.Situation;
import eu.datex2.schema.dynamisch.baustellen.SituationPublication;

/**
 * Konvertierung MDM-Baustellen in das DATEX-II Format.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */

public class Baustellen extends StandardKonvertierungDynamisch {

    /** Akzeptierte Datenverteiler-Objekttypen. */
    private static final String TYP_BAUSTELLEN = "typ.mdmBaustelle";

    /** Akzeptierte Datenverteiler-Attributgruppen. */
    private static final String ATG_BAUSTELLEN = "atg.mdmBaustellenDatenGesamt";

    /**
     * ArbeitsstellenIDExportieren: <ja | nein>: legt fest, ob die Arbeitsstellen-ID
     * exportiert werden soll oder nicht.
     */
    private static final String ARBEITSSTELLENIDEXPORTIEREN = "ArbeitsstellenIDExportieren";

    /**
     * Konstruktor.
     * 
     * @param plugin Referenz auf Plugin-Definitions-Klasse.
     */

    public Baustellen(final PluginExport plugin) {
        this.plugin = plugin;
    }

    /***************************************************************************
     * IN DEM FOLGENDEN ABSCHNITT WERDEN STRUKTURELL GLEICHE METHODEN GESAMMELT.
     * 
     * Diese Methoden werden in den einzelnen Plugins wiederholt - allerdings
     * beziehen sich die JAXB-Elemente trotz gleichen Namens dann auf andere
     * Klassen; eine nominale Gleichheit existiert also nicht.
     ***************************************************************************
     */

    /**
     * Gebe den in dieser Klasse verwendeten JAXBContext zur&uuml;ck.
     * 
     * @throws JAXBException JAXBException
     * @see javax.xml.bind.JAXBContext#newInstance(String)
     * @return JAXBContext Verwendeter Kontext
     */

    @Override
    public JAXBContext getJAXBContext() throws JAXBException {
        if (jaxbContext == null) {
            jaxbContext = JAXBContext.newInstance(D2LogicalModel.class.getPackage().getName());
        }
        return jaxbContext;
    }

    /**
     * Setzt die allgemeinen Angaben zur Publikation (Exchange, Language,
     * PublicationCreator).
     * 
     * @param d2l     D2LogicalModel
     * @param payload Verwendeter Payload
     */

    private void setzeNationalIdentifier(final ObjectFactory objectFactory, final D2LogicalModel d2l,
            final PayloadPublication payload) {
        final InternationalIdentifier internationalIdentifier = getInternationalIdentifier(objectFactory);
        // Setze Exchange
        final Exchange exchange = new Exchange();
        exchange.setSupplierIdentification(internationalIdentifier);
        d2l.setExchange(exchange);

        // Setze Language
        payload.setLang("DE");

        payload.setPublicationCreator(internationalIdentifier);
    }

    private InternationalIdentifier getInternationalIdentifier(final ObjectFactory objectFactory) {
        final InternationalIdentifier internationalIdentifier = objectFactory.createInternationalIdentifier();
        internationalIdentifier.setCountry(CountryEnum.DE);
        internationalIdentifier.setNationalIdentifier(nationalIdentifier);
        return internationalIdentifier;
    }

    /***********************************************************************
     * ENDE DES ABSCHNITTS STRUKTURELL GLEICHER METHODEN
     ***********************************************************************
     */

    /** {@inheritDoc} */
    @Override
    public Object getDatexII(final Map<ObjekteAtgAsp, Collection<ResultData>> data) {
        final Map<String, Datex2Situation> idmap = new HashMap<>();

        // Initialize Factories
        final ObjectFactory objectFactory = new ObjectFactory();
        final D2LogicalModel d2l = objectFactory.createD2LogicalModel();

        // Schema-Angaben
        d2l.setExtensionName(d2l.getExtensionName());
        d2l.setExtensionVersion(d2l.getExtensionVersion());
        d2l.setModelBaseVersion(d2l.getModelBaseVersion());

        final SituationPublication payload = objectFactory.createSituationPublication();

        setzeNationalIdentifier(objectFactory, d2l, payload);

        payload.setLang("DE");
        payload.setPublicationCreator(getInternationalIdentifier(objectFactory));

        final XMLGregorianCalendar xmlGregorianCalendar = WerkzeugSammlung.konvertiereDatum(new Date());
        payload.setPublicationTime(xmlGregorianCalendar);

        final HeaderInformation headerInformation = objectFactory.createHeaderInformation();
        headerInformation.setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);
        headerInformation.setInformationStatus(InformationStatusEnum.REAL);

        for (ObjekteAtgAsp objekteAtgAsp : data.keySet()) {
            if (!objekteAtgAsp.getAttributGruppe().getPid().equals(ATG_BAUSTELLEN)) {
                LOGGER.warning(logPrefix + "Unpassende Konfiguration des Plugins: "
                        + "Typ/Attributgruppe passt nicht zur dynamischen Konvertierung");
            } else {
                for (ResultData resultData : data.get(objekteAtgAsp)) {
                    if ((resultData != null) && (resultData.hasData())) {
                        if (!resultData.getDataDescription().getAttributeGroup().getPid().equals(ATG_BAUSTELLEN)
                                || !resultData.getObject().getType().getPid().equals(TYP_BAUSTELLEN)) {
                            LOGGER.warning(logPrefix + "Unpassende Konfiguration des Plugins: " + "Typ/Attributgruppe ("
                                    + resultData.getObject().getType().getPid() + "/"
                                    + resultData.getDataDescription().getAttributeGroup().getPid()
                                    + ") passt nicht zur dynamischen Konvertierung");
                            continue;
                        }
                        final Datex2Situation datex2situation = new Datex2Situation(resultData, objectFactory);
                        try {
                            final Situation situation = datex2situation.getSituation();

                            // pruefe Eindeutigkeit der IDs
                            final String id = situation.getId();
                            final Datex2Situation datex2SituationMap = idmap.get(id);
                            if (datex2SituationMap != null) {
                                LOGGER.warning(logPrefix + "Doppelte Situations-ID: '" + id + "' bei Baustelle '"
                                        + resultData.getObject().getPid() + "' (gleiche ID wie Baustelle '"
                                        + datex2SituationMap.getResultData().getObject().getPid()
                                        + "'\nBaustelle wird nicht exportiert!");
                                continue;
                            }

                            idmap.put(id, datex2situation);
                            payload.getSituation().add(situation);
                        } catch (final Exception ex) {
                            LOGGER.warning(logPrefix + "Verwerfe Baustelle '" + resultData.getObject().getPid()
                                    + "' wegen Fehler in der Verarbeitung", ex);
                        }

                    } // resultDate == null
                } // end for
            }
        }

        d2l.setPayloadPublication(payload);
        return d2l;
    }

    @Override
    public void initialisiere(final Collection<ObjekteAtgAsp> objects) throws ConfigurationException {
        // initialisiereBasisFelder();
        //
        // initialisiereStandardIntervall();
        logPrefix = WerkzeugSammlung.erzeugeLogPrefix(plugin);
        nationalIdentifier = WerkzeugSammlung.holeKonfigurationsdatum(plugin, KEY_NATIONAL_IDENTIFIER);
        try {
            final String idexportieren = WerkzeugSammlung.holeKonfigurationsdatum(plugin, ARBEITSSTELLENIDEXPORTIEREN);
            if ((idexportieren != null) && "Ja".equalsIgnoreCase(idexportieren)) {
                ExtraKonfiguration.setArbeitsstellenIDExportieren(true);
            } else {
                ExtraKonfiguration.setArbeitsstellenIDExportieren(false);
            }
        } catch (final javax.naming.ConfigurationException ce) {
            // ArbeitsstellenIDExportieren nicht in Konfiguration
            ExtraKonfiguration.setArbeitsstellenIDExportieren(false);
        }
        // ExtraKonfiguration.setLogger(LOGGER);
        // ExtraKonfiguration.setLogPrefix(logPrefix);
    }
}
