/*
 * Plug-in KEx DATEX II - Baustellen
 * Copyright (C) 2018 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente.Datex2ArtDerMassnahmeEnum;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.elemente.Datex2ArtDerMassnahmeKnotenZusatzEnum;
import eu.datex2.schema.dynamisch.baustellen.AffectedCarriagewayAndLanes;
import eu.datex2.schema.dynamisch.baustellen.CarriagewayEnum;
import eu.datex2.schema.dynamisch.baustellen.NetworkLocation;
import eu.datex2.schema.dynamisch.baustellen.RoadOrCarriagewayOrLaneManagement;
import eu.datex2.schema.dynamisch.baustellen.RoadOrCarriagewayOrLaneManagementTypeEnum;
import eu.datex2.schema.dynamisch.baustellen.SituationRecord;
import eu.datex2.schema.dynamisch.baustellen.SupplementaryPositionalDescription;

/**
 * Wrapper f&uuml;r {@link RoadOrCarriagewayOrLaneManagement}.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */
public class Datex2RoadOrCarriagewayOrLaneManagement extends Datex2NetworkManagement {

	/**
	 * Konstruktor.
	 * 
	 * @param situationRecord
	 *            {@link SituationRecord}
	 * @param attprefix
	 *            Attribut-Prefix
	 * @param resultData
	 *            {@link ResultData}
	 * @param data
	 *            {@link Data}
	 */
	public Datex2RoadOrCarriagewayOrLaneManagement(final SituationRecord situationRecord, final String attprefix,
			final ResultData resultData, final Data data) {
		super(situationRecord, attprefix, resultData, data);
	}

	@Override
	public SituationRecord getSituationRecord() throws Datex2BaustelleException {
		final SituationRecord situationRecord = super.getSituationRecord();

		// roadOrCarriagewayOrLaneManagementType (ueber actionPlanIdentifier)
		final Datex2ArtDerMassnahmeEnum datex2ArtDerMassnahmeEnum = getDatex2ArtDerMassnahmeEnum();
		final RoadOrCarriagewayOrLaneManagement ro = (RoadOrCarriagewayOrLaneManagement) situationRecord;
		ro.setRoadOrCarriagewayOrLaneManagementType(
				datex2ArtDerMassnahmeEnum.getRoadOrCarriagewayOrLaneManagementTypeEnum());

		// Die Zusatzinformationen der Sperrungen sind im DatK nur für die
		// Bauabschnitte modelliert
		if (isBauabschnitt() && isSperrung(ro)) {
			final int intValue = getData().getItem("BaArtDerMaßnahmeKnotenZusatz").asUnscaledValue().intValue();
			final Datex2ArtDerMassnahmeKnotenZusatzEnum admzus = Datex2ArtDerMassnahmeKnotenZusatzEnum
					.getInstance(intValue);

			final List<CarriagewayEnum> carriageways = new ArrayList<>();

			// siehe Abbildung 28
			switch (admzus) {
			case KeinKnoten: // Sperrung Richtungsfahrbahn
				carriageways.add(CarriagewayEnum.MAIN_CARRIAGEWAY);
				break;

			case Einfahrt:
				carriageways.add(CarriagewayEnum.ENTRY_SLIP_ROAD);
				break;

			case Ausfahrt:
				carriageways.add(CarriagewayEnum.EXIT_SLIP_ROAD);
				break;

			case EinUndAusfahrt:
				carriageways.add(CarriagewayEnum.SLIP_ROADS);
				break;

			case Parallelfahrbahn:
				carriageways.add(CarriagewayEnum.PARALLEL_CARRIAGEWAY);
				break;

			case Verbindungsfahrbahn:
				carriageways.add(CarriagewayEnum.CONNECTING_CARRIAGEWAY);
				break;

			case VerbindungsUndParallelfahrbahn:
				carriageways.add(CarriagewayEnum.PARALLEL_CARRIAGEWAY);
				carriageways.add(CarriagewayEnum.CONNECTING_CARRIAGEWAY);
				break;
			default:
				throw new Datex2BaustelleException(
						"Nicht unterstützter Wert für 'BaArtDerMaßnahmeKnotenZusatz': " + admzus.toString());
			}

			final SupplementaryPositionalDescription spd = getObjectFactory()
					.createSupplementaryPositionalDescription();

			for (CarriagewayEnum ce : carriageways) {
				final AffectedCarriagewayAndLanes ac = getObjectFactory().createAffectedCarriagewayAndLanes();
				ac.setCarriageway(ce);
				spd.getAffectedCarriagewayAndLanes().add(ac);
			}

			final Array array = getData().getArray("BaKnotenZusatzLageDerRampe");
			if (array.getLength() > 0) {
				spd.setSequentialRampNumber(BigInteger.valueOf(array.getItem(0).asUnscaledValue().longValue()));
			}

			final NetworkLocation nl = (NetworkLocation) (situationRecord.getGroupOfLocations());
			nl.setSupplementaryPositionalDescription(spd);
		}

		return situationRecord;
	}

	private boolean isSperrung(final RoadOrCarriagewayOrLaneManagement ro) {
		final RoadOrCarriagewayOrLaneManagementTypeEnum roadOrCarriagewayOrLaneManagementType = ro
				.getRoadOrCarriagewayOrLaneManagementType();
		return roadOrCarriagewayOrLaneManagementType == RoadOrCarriagewayOrLaneManagementTypeEnum.CARRIAGEWAY_CLOSURES
				|| roadOrCarriagewayOrLaneManagementType == RoadOrCarriagewayOrLaneManagementTypeEnum.ROAD_CLOSED;
	}
}
