/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.konvertierung.impl;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.Map;

import javax.naming.ConfigurationException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.datatype.XMLGregorianCalendar;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.PayloadPublication;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.ConfidentialityValueEnum;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.CountryEnum;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.D2LogicalModel;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.ElaboratedData;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.ElaboratedDataPublication;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.Exchange;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.HeaderInformation;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.InformationStatusEnum;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.InternationalIdentifier;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.LocationByReference;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.OverallPeriod;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.PredefinedLocationVersionedReference;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.PredefinedNonOrderedLocationGroupVersionedReference;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.ReferenceSettings;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.TrafficStatus;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.TrafficStatusEnum;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.TrafficStatusValue;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.Validity;
import eu.datex2.schema.dynamisch.messquerschnitte.verkehrslage.ValidityStatusEnum;

/**
 * Konvertierung von dynamischen Verkehrslagedaten in das DATEX-II Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class MQVerkehrslage extends StandardKonvertierungDynamisch {
  /** Schlüssel die Veröffentlichung von freiem Verkehr. */
  private static final String FREIEN_VERKEHR_VEROEFFENTLICHEN = "FreienVerkehrVeroeffentlichen";

  /** Faktor zur Umrechnung Sekunde => Millisekunde. */
  private static final int FACTOR_MILLI_TO_SECOND = 1000;

  /** Faktor zur Umrechnung [0;1] => Prozent [0;100]. */
  private static final int FACTOR_PERCENT = 100;

  /** Akzeptierter Datenverteiler-Typ. */
  private static final String TYP_MESSQUERSCHNITT = "typ.messQuerschnitt";

  /** Akzeptierte Datenverteiler-Attributgruppe. */
  private static final String ATG_VERKEHRSDATEN = "atg.störfallZustand";

  private boolean freienVerkehrVeroeffentlichen;

  /**
   * Konstruktor.
   * 
   * @param plugin Referenz auf Plugin-Definitions-Klasse.
   */
  public MQVerkehrslage(final PluginExport plugin) {
    this.plugin = plugin;
  }

  /***************************************************************************
   * IN DEM FOLGENDEN ABSCHNITT WERDEN STRUKTURELL GLEICHE METHODEN GESAMMELT.
   * 
   * Diese Methoden werden in den einzelnen Plugins wiederholt - allerdings
   * beziehen sich die JAXB-Elemente trotz gleichen Namens dann auf andere
   * Klassen; eine nominale Gleichheit existiert also nicht.
   ***************************************************************************
   */

  /**
   * Gebe den in dieser Klasse verwendeten JAXBContext zurück.
   * 
   * @throws JAXBException JAXBException
   * @see javax.xml.bind.JAXBContext#newInstance(String)
   * @return JAXBContext Verwendeter Kontext
   */

  @Override
  public JAXBContext getJAXBContext() throws JAXBException {
    if (jaxbContext == null) {
      jaxbContext = JAXBContext.newInstance(D2LogicalModel.class.getPackage().getName());
    }
    return jaxbContext;
  }

  /**
   * Setzt die allgemeinen Angaben zur Publikation (Exchange, Language,
   * PublicationCreator).
   * 
   * @param d2l D2LogicalModel
   * @param payload Verwendeter Payload
   */

  private void setzeNationalIdentifier(final D2LogicalModel d2l, final PayloadPublication payload) {
    InternationalIdentifier internationalIdentifier = new InternationalIdentifier();
    internationalIdentifier.setCountry(CountryEnum.DE);
    internationalIdentifier.setNationalIdentifier(nationalIdentifier);

    // Setze Exchange

    Exchange exchange = new Exchange();
    exchange.setSupplierIdentification(internationalIdentifier);
    d2l.setExchange(exchange);

    // Setze Language
    payload.setLang("DE");

    payload.setPublicationCreator(internationalIdentifier);
  }

  /***********************************************************************
   * ENDE DES ABSCHNITTS STRUKTURELL GLEICHER METHODEN
   ***********************************************************************
   */

  /** {@inheritDoc} */
  @Override
  public Object getDatexII(final Map<ObjekteAtgAsp, Collection<ResultData>> data) {
    String predefinedLocationVersion = holePredefinedLocationVersion();
    D2LogicalModel d2l = new D2LogicalModel();
    ElaboratedDataPublication payload = new ElaboratedDataPublication();
    d2l.setPayloadPublication(payload);

    setzeNationalIdentifier(d2l, payload);

    setzeHeaderInformation(payload);

    setzeReferenceSettings(predefinedLocationVersion, payload);

    XMLGregorianCalendar xmlGregorianCalendar = WerkzeugSammlung.konvertiereDatum(new Date());
    payload.setPublicationTime(xmlGregorianCalendar);

    int anzahlStoerung = 0;
    int anzahlKeineAussage = 0;
    int anzahlFreierVerkehr = 0;
    int anzahlVerarbeitungsFehler = 0;
    for (ObjekteAtgAsp objekteAtgAsp : data.keySet()) {
      if (!objekteAtgAsp.getTyp().getPid().equals(TYP_MESSQUERSCHNITT)
          || (!objekteAtgAsp.getAttributGruppe().getPid().equals(ATG_VERKEHRSDATEN))) {
        LOGGER.warning(logPrefix + "Unpassende Konfiguration des Plugins: "
            + "Typ/Attributgruppe passt nicht zur dynamischen Konvertierung");
      } else {
        for (ResultData resultData : data.get(objekteAtgAsp)) {
          try {
            LocationByReference locationByReference = new LocationByReference();
            PredefinedLocationVersionedReference predefinedLocationVersionedReference = new PredefinedLocationVersionedReference();
            predefinedLocationVersionedReference.setId(resultData.getObject().getPid());
            predefinedLocationVersionedReference.setVersion(predefinedLocationVersion);
            locationByReference.setPredefinedLocationReference(predefinedLocationVersionedReference);
            ElaboratedData elaboratedData = new ElaboratedData();
            TrafficStatus trafficStatus = new TrafficStatus();

            if (resultData.hasData()) {
              boolean hasElaboratedData = false;
              String situation = resultData.getData().getTextValue("Situation").getValueText();
              long t = resultData.getData().getTimeValue("T").getMillis();
              float period = t / FACTOR_MILLI_TO_SECOND;

              long horizont = resultData.getData().getTimeValue("Horizont").getMillis();
              int horizontMilliSekunden = WerkzeugSammlung.getIntValue(horizont);

              float gueteIndex = WerkzeugSammlung.getFloatValue(resultData, "Güte", "Index", SMALL_MINUS_FLOAT, 2);
              Float gueteProzent = null;
              if (gueteIndex >= 0) {
                gueteProzent = Float.valueOf((1.0f - gueteIndex) * FACTOR_PERCENT);
              }

              trafficStatus.setPertinentLocation(locationByReference);
              trafficStatus.setMeasurementOrCalculationTime(xmlGregorianCalendar);
              trafficStatus.setMeasurementOrCalculationPeriod(period);

              TrafficStatusValue trafficStatusValue = new TrafficStatusValue();
              trafficStatus.setTrafficStatus(trafficStatusValue);

              trafficStatusValue.setAccuracy(gueteProzent);

              switch (situation.trim().toLowerCase()) {
              case "störung":
                trafficStatusValue.setDataError(true);
                trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.UNKNOWN);
                hasElaboratedData = false; // true
                anzahlStoerung++;
                LOGGER.fine(logPrefix + "Wert wegen 'störung' nicht veröffentlicht für Referenz-Location "
                    + locationByReference.getPredefinedLocationReference().getId());
                break;
              case "keine aussage":
                trafficStatusValue.setDataError(true);
                trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.UNKNOWN);
                hasElaboratedData = false; // true
                anzahlKeineAussage++;
                LOGGER.fine(logPrefix + "Wert wegen 'keine aussage' nicht veröffentlicht für Referenz-Location "
                    + locationByReference.getPredefinedLocationReference().getId());
                break;
              case "lebhafter verkehr":
              case "dichter verkehr":
                trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.HEAVY);
                hasElaboratedData = true;
                break;
              case "zähfließender verkehr":
              case "stockender verkehr":
                trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.CONGESTED);
                hasElaboratedData = true;
                break;
              case "stau":
                trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.IMPOSSIBLE);
                hasElaboratedData = true;
                break;
              case "freier verkehr":
                if (payload.getElaboratedDatas().isEmpty()) {
                  trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.FREE_FLOW);
                  hasElaboratedData = true;
                } else {
                  if (freienVerkehrVeroeffentlichen) {
                    trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.FREE_FLOW);
                    hasElaboratedData = true;
                  } else {
                    // Defaultwert Freier Verkehr = TrafficStatusEnum.FREE_FLOW
                    LOGGER.finer(logPrefix
                        + "Wert 'freier verkehr' wegen 'FreienVerkehrVeroeffentlichen=false' nicht veröffentlicht für Referenz-Location "
                        + locationByReference.getPredefinedLocationReference().getId());
                    anzahlFreierVerkehr++;
                    hasElaboratedData = false;
                  }
                }
                break;
              default:
                trafficStatusValue.setDataError(true);
                trafficStatusValue.setTrafficStatusValue(TrafficStatusEnum.UNKNOWN);
                hasElaboratedData = false; // true
                break;
              }

              if (hasElaboratedData) {
                elaboratedData.setBasicData(trafficStatus);
                if (horizont > 0) {
                  elaboratedData.setValidity(setzeValidity(horizontMilliSekunden));
                  elaboratedData.setForecast(true);
                }
                payload.getElaboratedDatas().add(elaboratedData);
              } // hasElaboratedData
            }

          } catch (Exception ex) {
            LOGGER.fine(logPrefix + "Verwerfe einen Eintrag wegen Fehler in der Verarbeitung", ex);
            anzahlVerarbeitungsFehler++;
          }
        } // end for
      }
    } // end for

    LOGGER.info(logPrefix + String.format("Anzahl Werte 'störung': %d", anzahlStoerung));
    LOGGER.info(logPrefix + String.format("Anzahl Werte 'keine aussage': %d", anzahlKeineAussage));
    LOGGER.info(logPrefix + String.format("Anzahl Werte 'freier verkehr': %d", anzahlFreierVerkehr));
    LOGGER.info(logPrefix + String.format("Anzahl Verarbeitungsfehler: %d", anzahlVerarbeitungsFehler));
    LOGGER.info(logPrefix + String.format("Anzahl fehlerhafter Werte gesamt: %d",
        anzahlStoerung + anzahlKeineAussage + anzahlVerarbeitungsFehler));

    if (payload.getElaboratedDatas().isEmpty()) {
      LOGGER.warning(logPrefix + "Kein valider Datensatz verfügbar, Publikation wird abgebrochen");
      return null;
    } else {
      return d2l;
    }
  }

  /**
   * Setze allgemeine Referenzangaben (LocationGroupId, -Version etc.).
   * 
   * @param predefinedLocationVersion die zu verwendende Version
   * @param payload der zu modifizierende Payload
   */

  private void setzeReferenceSettings(final String predefinedLocationVersion, final ElaboratedDataPublication payload) {
    ReferenceSettings referenceSettings = new ReferenceSettings();
    PredefinedNonOrderedLocationGroupVersionedReference predefinedNonOrderedLocationGroupVersionedReference = new PredefinedNonOrderedLocationGroupVersionedReference();
    predefinedNonOrderedLocationGroupVersionedReference.setId(locationGroupId);
    predefinedNonOrderedLocationGroupVersionedReference.setVersion(predefinedLocationVersion);
    referenceSettings
        .setPredefinedNonOrderedLocationGroupReference(predefinedNonOrderedLocationGroupVersionedReference);
    referenceSettings.setTrafficStatusDefault(TrafficStatusEnum.FREE_FLOW);
    payload.setReferenceSettings(referenceSettings);
  }

  /**
   * Setze die HeaderInformation des übergebenen Payloads.
   * 
   * @param payload Zu ergänzender Payload
   */

  private void setzeHeaderInformation(final ElaboratedDataPublication payload) {
    HeaderInformation headerInformation = new HeaderInformation();
    headerInformation.setConfidentiality(ConfidentialityValueEnum.NO_RESTRICTION);
    headerInformation.setInformationStatus(InformationStatusEnum.REAL);
    payload.setHeaderInformation(headerInformation);
  }

  /**
   * Setzt ein Start und Enddatum in ein DatexII Validity-Objekt.
   * 
   * @param horizont Ineger Millisekunden als Intervalldauer
   * @return Validity
   */

  protected Validity setzeValidity(final int horizont) {
    Validity validity = new Validity();
    validity.setValidityStatus(ValidityStatusEnum.DEFINED_BY_VALIDITY_TIME_SPEC);
    OverallPeriod overallPeriod = new OverallPeriod();
    Date datumStart = new Date();
    Calendar kalender = Calendar.getInstance();
    kalender.setTime(datumStart);
    kalender.add(Calendar.MILLISECOND, horizont);
    Date datumEnd = kalender.getTime();

    XMLGregorianCalendar xmlGregorianCalendarStart = WerkzeugSammlung.konvertiereDatum(datumStart);
    XMLGregorianCalendar xmlGregorianCalendarEnd = WerkzeugSammlung.konvertiereDatum(datumEnd);

    overallPeriod.setOverallStartTime(xmlGregorianCalendarStart);
    overallPeriod.setOverallEndTime(xmlGregorianCalendarEnd);
    validity.setValidityTimeSpecification(overallPeriod);
    return validity;
  }

  @Override
  public void initialisiere(final Collection<ObjekteAtgAsp> objects) throws ConfigurationException {
    String tmpFreienVerkehrVeroeffentlichen;
    try {
      tmpFreienVerkehrVeroeffentlichen = WerkzeugSammlung.holeKonfigurationsdatum(plugin,
          FREIEN_VERKEHR_VEROEFFENTLICHEN);
      if ("true".equalsIgnoreCase(tmpFreienVerkehrVeroeffentlichen)) {
        freienVerkehrVeroeffentlichen = true;
      } else if ("false".equalsIgnoreCase(tmpFreienVerkehrVeroeffentlichen)) {
        freienVerkehrVeroeffentlichen = false;
      } else {
        LOGGER.warning(logPrefix
            + "Kein valider Wert ('true' oder 'false') für 'FreienVerkehrVeroeffentlichen' in Konfigurationsdatei gesetzt. Es wird 'false' angenommen.");
      }
    } catch (ConfigurationException e) {
      LOGGER.warning(logPrefix
          + "Kein Konfigurationsparameter 'FreienVerkehrVeroeffentlichen' in Konfigurationsdatei gesetzt. Es für wird 'FreienVerkehrVeroeffentlichen' 'false' angenommen.");
      freienVerkehrVeroeffentlichen = false;
    }

    initialisiereBasisFelder();

    initialisiereStandardIntervall();
  }

}
